﻿using CsvUploader.Model;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CsvUploader
{
    class CsvUploadTaskService
    {
        public const string TaskSettingFileName = "Task.ini";

        public CsvUploadTaskService(string settingFileName = TaskSettingFileName)
        {
            if (!File.Exists(settingFileName))
            {
                throw new FileNotFoundException("Task setting file Not Found!", settingFileName, null);
            }

            var taskSetting = JsonConvert.DeserializeObject<UploadTask[]>(File.ReadAllText(settingFileName));

            if (taskSetting is null)
            {
                throw new Exception("Task setting file Formate Error");
            }

            InitBase(taskSetting);
        }

        public CsvUploadTaskService(UploadTask[] uploadTaskSetting)
        {
            InitBase(uploadTaskSetting);
        }

        private void InitBase(UploadTask[] uploadTaskSetting)
        {
            if (uploadTaskSetting is null)
            {
                throw new NullReferenceException("Task setting should not be null");
            }

            CheckNullSetting(uploadTaskSetting);
            //CheckColumeMatch(uploadTaskSetting);

            StartTasks(uploadTaskSetting);
        }

        private void StartTasks(UploadTask[] uploadTaskSetting)
        {
            foreach(var taskSetting in uploadTaskSetting)
            {
                var task = new CsvUploadTask(taskSetting);
                task.Start();
            }
        }

        private static void CheckNullSetting(UploadTask[] uploadTaskSetting)
        {
            for(int index = 0; index < uploadTaskSetting.Count(); index++)
            {
                var task = uploadTaskSetting[index];
                if (string.IsNullOrEmpty(task.Name))
                {
                    throw new Exception($"Task NO,{index} Name should not be empty");
                }

                if (string.IsNullOrEmpty(task.Folder))
                {
                    throw new Exception($"Task {task.Name} file directory should not be empty");
                }

                if (!Directory.Exists(task.Folder))
                {
                    throw new Exception($"Task {task.Name} file directory not exist");
                }

                if (string.IsNullOrEmpty(task.SqlConnectionString))
                {
                    throw new Exception($"Task {task.Name} SqlConnectionString should not be empty");
                }

                //if (string.IsNullOrEmpty(task.IndexedColumnName))
                //{
                //    throw new Exception($"Task {task.Name} IndexedColumnName should not be empty");
                //}

                if (string.IsNullOrEmpty(task.TableName) &&
                    (task.TableNameRules is null || string.IsNullOrEmpty(task.TableNameRuleColumn)))
                {
                    throw new Exception($"Task {task.Name} TableName should not be empty");
                }
            }
        }

        //private static void CheckColumeMatch(UploadTask[] uploadTaskSetting)
        //{
        //    foreach(var task in uploadTaskSetting)
        //    {
        //        if (task.CsvColumnNames.Count() != task.DbColumnNames.Count())
        //        {
        //            throw new Exception($"{task.Name} CsvColumnNames count should equal DbColumnNames");
        //        }
        //    }
        //}
    }
}
