﻿using CsvUploader.Func;
using CsvUploader.Model;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CsvUploader
{
    static class UploadDataParserExt
    {
        public static Dictionary<string, string> AddSimpleData(this Dictionary<string, string> toPairs, IEnumerable<string> dataLines)
        {
            foreach (var line in dataLines)
            {
                var rowDatas = line.Split(new[] { ',' }, 2);
                toPairs[rowDatas[0]] = rowDatas[1];
            }
            return toPairs;
        }

        public static Dictionary<string, string> AddTablesData(this Dictionary<string, string> toPairs, List<CsvTable> tables)
        {
            if (tables is null || tables.Count == 0)
            {
                return toPairs;
            }

            foreach (var table in tables)
            {
                table.Keys = table.Keys.ReplaceByRule(toPairs).ToList();
                table.Cols = table.Cols.ReplaceByRule(toPairs).ToList();
            }

            CheckKeysColeExist(toPairs, tables);
            CheckColsColeExist(toPairs, tables);

            foreach (var table in tables)
            {
                toPairs.AddTableData(table);
            }

            return toPairs;
        }

        public static Dictionary<string, string> CleanDataTail(this Dictionary<string, string> toPairs)
        {
            var keys = toPairs.Keys.ToList();
            foreach(var key in keys)
            {
                toPairs[key] = toPairs[key].Trim(',');
            }
            return toPairs;
        }

        public static Dictionary<string,string> TryValueConvert(this Dictionary<string, string> toPairs, List<string> passToIntColumns)
        {
            if (passToIntColumns is null)
            {
                return toPairs;
            }

            var keys = toPairs.Keys.ToList();
            foreach (var key in keys)
            {
                if (!passToIntColumns.Contains(key))
                {
                    continue;
                }

                if (toPairs[key].ToLower() == "pass")
                {
                    toPairs[key] = "1";
                }
                if (toPairs[key].ToLower() == "fail")
                {
                    toPairs[key] = "0";
                }
            }
            return toPairs;
        }

        public static Dictionary<string, string> AddCsvWithConvert(this Dictionary<string, string> destination, Dictionary<string, string> csvPairs, Dictionary<string, string> convertPairs)
        {
            foreach(var pair in csvPairs)
            {
                if (convertPairs.Keys.Contains(pair.Key))
                {
                    destination.Add(convertPairs[pair.Key], pair.Value);
                }
            }
            return destination;
        }

        public static Dictionary<string,string> AddStaticValues(this Dictionary<string, string> toPairs, Dictionary<string,string> fromPairs)
        {
            if (fromPairs == null)
            {
                return toPairs;
            }

            foreach(var pair in fromPairs)
            {
                toPairs[pair.Key] = pair.Value;
            }
            return toPairs;
        }

        private static Dictionary<string, string> AddTableData(this Dictionary<string, string> toPairs, CsvTable table)
        {
            var keyCnt = table.Keys.Count;
            var sampleKeyRow = toPairs[table.Keys[0]];
            var dataCnt = sampleKeyRow.Count(x => x == ',') + 1;

            string[][] keyTable = new string[dataCnt][];
            for(int index = 0;index < keyTable.Length; index ++)
            {
                keyTable[index] = new string[keyCnt];
            }

            for (int keyIndex = 0; keyIndex < table.Keys.Count; keyIndex++)
            {
                var keyRow = toPairs[table.Keys[keyIndex]];
                var keyDatas = keyRow.Split(',');
                for (int keyDataIndex = 0; keyDataIndex < keyDatas.Count(); keyDataIndex++)
                {
                    keyTable[keyDataIndex][keyIndex] = keyDatas[keyDataIndex];
                }
            }

            var keyList = keyTable.Select(x => string.Join(".", x)).ToList();

            foreach (var col in table.Cols)
            {
                toPairs.AddTableColData(keyList, col);
            }
            return toPairs;
        }

        private static Dictionary<string, string> AddTableColData(this Dictionary<string, string> toPairs, List<string> keyList, string colName)
        {
            var colRow = toPairs[colName];
            var colDatas = colRow.Split(',');

            if (keyList.Count != colDatas.Length)
            {
                throw new Exception("key and data length not match");
            }

            for (int index = 0; index < colDatas.Length; index++)
            {
                var key = keyList[index];
                var generatedKey = $"{key}.{colName}";
                toPairs[generatedKey] = colDatas[index];
            }

            return toPairs;
        }

        private static void CheckKeysColeExist(Dictionary<string, string> toPairs, List<CsvTable> tables)
        {
            var keys = toPairs.Keys;
            foreach(var table in tables)
            {
                foreach (var key in table.Keys)
                {
                    if (!keys.Contains(key))
                    {
                        throw new Exception($"{table.Name} with key {key} not found!");
                    }
                }
            }
        }

        private static void CheckColsColeExist(Dictionary<string, string> toPairs, List<CsvTable> tables)
        {
            var keys = toPairs.Keys;
            foreach(var table in tables)
            {
                foreach (var col in table.Cols)
                {
                    if (!keys.Contains(col))
                    {
                        throw new Exception($"{table.Name} with col {col} not found!");
                    }
                }
            }
        }
    }
}
