﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CsvUploader
{
    class UploadRecordService
    {
        public const string UploadedRecordBase = "record";
        public string UploadedRecord => $"record_{_taskName}";
        private string _taskName;
        private Model.UploadedRecord _record;
        private List<Model.UploadedRecord> _records;

        public UploadRecordService(string taskName)
        {
            if (string.IsNullOrEmpty(taskName))
            {
                throw new Exception("task Name should not be empty");
            }
            _taskName = taskName;

            if (!File.Exists(UploadedRecord))
            {
                var stream = File.CreateText(UploadedRecord);
                stream.Write("[]");
                stream.Close();
                stream.Dispose();
            }

            var records = JsonConvert.DeserializeObject<List<Model.UploadedRecord>>(File.ReadAllText(UploadedRecord));

            if (records is null)
            {
                throw new Exception($"File {UploadedRecord} formate error");
            }

            var record = records.FirstOrDefault(x => x.TaskName == taskName);
            if (record == null)
            {
                record = new Model.UploadedRecord() {
                    TaskName = taskName,
                    FileNames = new List<string>(), 
                    FileUploadDate = new List<DateTime>()
                };
                records.Add(record);
            }

            this._record = record;
            _records = records;
        }

        public DateTime? TryGetUploadDateTime(string fileName)
        {
            if (_record.FileNames.Contains(fileName))
            {
                var index = _record.FileNames.IndexOf(fileName);
                return _record.FileUploadDate[index];
            }
            return null;
        }

        public void SetUploadDateTime(string fileName,DateTime dateTime)
        {
            fileName = Path.GetFileName(fileName);
            if (_record.FileNames.Contains(fileName))
            {
                var index = _record.FileNames.IndexOf(fileName);
                _record.FileUploadDate[index] = dateTime;

                return;
            }

            _record.FileNames.Add(fileName);
            _record.FileUploadDate.Add(dateTime);

            File.WriteAllText(UploadedRecord, JsonConvert.SerializeObject(_records));
        }
    }
}
